/************************************************************************
 *   IRC - Internet Relay Chat, admin/flagparse.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "flagparse.h"
#include "irc_string.h"
#include "send.h"
#include "ircd.h"
#include "numeric.h"

#include <string.h>

flag
flag_parse(char *flag_string, struct flag_def defs[])
{
  char *p, *current;
  flag f = NULL_FLAG;
  struct flag_def *def;

  if (!flag_string)
    return NULL_FLAG;

  current = flag_string;
  while(current && *current)
    {
      /* Get rid of any superflous ,s */
      while (*current == ',')
	current++;
      if (*current == '\0')
	return f;
      p = strchr(current, ',');
      if (p)
	*p = '\0';
      /* OK, that delimits the start and end of current properly */
      for (def = defs; def->name; def++)
	if (!irccmp(def->name, current))
	  {
	    f |= def->value;
	    break;
	  }
      current = p;
    }

  return f;
}

char *
flag_string(flag flags, struct flag_def defs[])
{
  static char s[BUFSIZE];
  int len = 0;
  struct flag_def *def;

  s[0] = '\0';

  for (def = defs; def->name; def++)
    if (flags & def->value)
      {
	if ((len + strlen(def->name) + 1) > (BUFSIZE - 1))
	  break;
	strcat(s, def->name);
	strcat(s, ",");
	len += strlen(def->name) + 1;
      }

  /* Return a lone , if there are no flags */
  if (len == 0)
    {
      s[0] = ',';
      s[1] = '\0';
    }
  else
    /* But don't return a trailing , if there are any */
    {
      s[len - 1] = '\0';
    }

  return s;
}
